/************************************************************************
 *  ser.h - Simple Encoding Rules header
 ************************************************************************
 *  (c)2008 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _SER_H_
#define _SER_H_

#include <stdint.h>
#include <cutil/cutil_portable.h>


#define	SER_TYPE_INTEGER16	0x00100000
#define	SER_TYPE_INTEGER32	0x00200000
#define	SER_TYPE_INTEGER64	0x00400000
#define	SER_TYPE_FLOAT		0x00800000
#define SER_TYPE_STRING		0x01000000
#define SER_TYPE_BINARY		0x02000000
#define SER_TYPE_ARRAY		0x04000000
#define SER_TYPE_SIGNED		0x80000000

#define	SER_WHENCE_FIRST	1
#define	SER_WHENCE_LAST		2
#define	SER_WHENCE_NEXT		3
#define	SER_WHENCE_PREVIOUS	4


typedef struct	_SERElement	SERElement;
typedef struct	_SERPacket	SERPacket;


CUTIL_API int ser_element_new (SERElement **sep, int type, void *valuep, int length);
CUTIL_API int ser_element_free (SERElement **sep);
CUTIL_API int ser_element_new_uint16 (SERElement **sep, uint16_t value);
CUTIL_API int ser_element_new_int16 (SERElement **sep, int16_t value);
CUTIL_API int ser_element_new_uint32 (SERElement **sep, uint32_t value);
CUTIL_API int ser_element_new_int32 (SERElement **sep, int32_t value);
CUTIL_API int ser_element_new_uint64 (SERElement **sep, uint64_t value);
CUTIL_API int ser_element_new_int64 (SERElement **sep, int64_t value);
CUTIL_API int ser_element_new_float (SERElement **sep, float value);
CUTIL_API int ser_element_new_string (SERElement **sep, char *value);
CUTIL_API int ser_element_new_string_length (SERElement **sep, char *value, int length);
CUTIL_API int ser_element_new_binary (SERElement **sep, char *value, int len);
/* CUTIL_API int ser_element_new_array (SERElement **sep); */
CUTIL_API int ser_element_get_type (SERElement *se, uint32_t *typep);
CUTIL_API int ser_element_get_value (SERElement *se, void *valuep, int *lengthp);

CUTIL_API int ser_packet_new (SERPacket **spp);
CUTIL_API int ser_packet_free (SERPacket **spp);
CUTIL_API int ser_packet_clear (SERPacket *sp);
CUTIL_API int ser_packet_append (SERPacket *sp, SERElement *se);
CUTIL_API int ser_packet_prepend (SERPacket *sp, SERElement *se);
CUTIL_API int ser_packet_serialize (SERPacket *sp, char *buffer, int *lenp);
CUTIL_API int ser_packet_deserialize (SERPacket **spp, char *buffer, int len);
CUTIL_API SERElement *ser_packet_iterate_elements (SERPacket *sp, uint32_t whence);
CUTIL_API int ser_packet_length (SERPacket *sp, uint32_t *lengthp);
CUTIL_API int ser_packet_elements (SERPacket *sp, uint32_t *countp);
CUTIL_API int ser_packet_append_as_array (SERPacket *sp, SERPacket *ssp);
CUTIL_API int ser_packet_new_with_format (SERPacket **spp, char *format, ...);
CUTIL_API int ser_packet_get_elements_with_format (SERPacket *sp, char *format, ...);


/* Caching functions */
CUTIL_API int ser_packet_new_element_c (SERPacket *sp, SERElement **sep,
				int type, void *valuep, int length);
CUTIL_API int ser_packet_deserialize_c (SERPacket *sp, char *buffer, int len);
CUTIL_API int ser_packet_clear_c (SERPacket *sp);


#endif

