/************************************************************************/
/* xmlrpc.h - XMLRPC Implementation Header				*/
/************************************************************************/
/*
 *  (c)2003 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/


#ifndef _XMLRPC_H_
#define _XMLRPC_H_

#include <cutil/cutil_portable.h>

#define XMLRPC_VALUETYPE_INT		0x01
#define XMLRPC_VALUETYPE_BOOL		0x02
#define XMLRPC_VALUETYPE_STRING		0x03
#define XMLRPC_VALUETYPE_DOUBLE		0x04
#define XMLRPC_VALUETYPE_DATETIME	0x05
#define XMLRPC_VALUETYPE_BASE64		0x06
#define XMLRPC_VALUETYPE_STRUCT		0x07
#define XMLRPC_VALUETYPE_ARRAY		0x08


/* XMLRPC Opaque Primitives */
typedef struct _XmlrpcValue XmlrpcValue;
typedef struct _XmlrpcStruct XmlrpcStruct;
typedef struct _XmlrpcArray XmlrpcArray;
typedef struct _XmlrpcRequest XmlrpcRequest;
typedef struct _XmlrpcResponse XmlrpcResponse;



/* XMLRPC Reference Counting functions */
CUTIL_API int xmlrpc_value_ref_inc (XmlrpcValue *xv);
CUTIL_API int xmlrpc_value_ref_dec (XmlrpcValue **xvp);
CUTIL_API int xmlrpc_struct_ref_inc (XmlrpcStruct *xs);
CUTIL_API int xmlrpc_struct_ref_dec (XmlrpcStruct **xsp);
CUTIL_API int xmlrpc_array_ref_inc (XmlrpcArray *xa);
CUTIL_API int xmlrpc_array_ref_dec (XmlrpcArray **xap);


/* XMLRPC value functions */
CUTIL_API int xmlrpc_value_new (XmlrpcValue **xvp, int valuetype,
			void *value, int valuelen);
CUTIL_API int xmlrpc_value_new_from_struct (XmlrpcValue **xvp, XmlrpcStruct *xs);
CUTIL_API int xmlrpc_value_new_from_array (XmlrpcValue **xvp, XmlrpcArray *xa);
CUTIL_API int xmlrpc_value_new_fault (XmlrpcValue **xvp,
			int faultcode, char *faultstring);

CUTIL_API int xmlrpc_value_get_value (XmlrpcValue *xv, int *valuetype, void **valuep,
			int *valuelen);
CUTIL_API int xmlrpc_value_get_struct (XmlrpcValue *xv, XmlrpcStruct **xsp);
CUTIL_API int xmlrpc_value_get_array (XmlrpcValue *xv, XmlrpcArray **xap);


/* XMLRPC struct functions */
CUTIL_API int xmlrpc_struct_new (XmlrpcStruct **xsp);
CUTIL_API int xmlrpc_struct_add_member (XmlrpcStruct *xs, char *membername,
				XmlrpcValue *membervalue);
CUTIL_API int xmlrpc_struct_member_count (XmlrpcStruct *xs, int *membercount);
CUTIL_API int xmlrpc_struct_get_member (XmlrpcStruct *xs, char *membername,
				XmlrpcValue **membervalue);
CUTIL_API int xmlrpc_struct_has_member (XmlrpcStruct *xs, char *membername, int *hasmember);
CUTIL_API int xmlrpc_struct_remove_member (XmlrpcStruct *xs, char *membername);

/* XMLRPC array functions */
CUTIL_API int xmlrpc_array_new (XmlrpcArray **xap);
CUTIL_API int xmlrpc_array_add_value (XmlrpcArray *xa, XmlrpcValue *value);
CUTIL_API int xmlrpc_array_value_count (XmlrpcArray *xa, int *valuecount);
CUTIL_API int xmlrpc_array_get_value (XmlrpcArray *xa, int valuenumber,
				XmlrpcValue **xvp);

/* XMLRPC request functions */
CUTIL_API int xmlrpc_request_new (XmlrpcRequest **xrp, char *methodname);
CUTIL_API int xmlrpc_request_add_param (XmlrpcRequest *xr, XmlrpcValue *param);
CUTIL_API int xmlrpc_request_get_param_count (XmlrpcRequest *xr, int *paramcount);
CUTIL_API int xmlrpc_request_get_param (XmlrpcRequest *xr, int paramnumber,
			XmlrpcValue **xvp);
CUTIL_API int xmlrpc_request_get_methodname (XmlrpcRequest *xr, const char **methodnamep);
CUTIL_API int xmlrpc_request_serialize (XmlrpcRequest *xr, char **outputxmlp,
			int *outputxmllen);
CUTIL_API int xmlrpc_request_parse_xml (XmlrpcRequest **xrp, char *inputxml,
			int inputxmllen);
CUTIL_API int xmlrpc_request_free (XmlrpcRequest **xrp);

/* XMLRPC response functions */
CUTIL_API int xmlrpc_response_new (XmlrpcResponse **xrp, char *methodname,
			XmlrpcValue *xv);
CUTIL_API int xmlrpc_response_fault_new (XmlrpcResponse **xrp, char *methodname,
			int faultcode, char *faultstring);
CUTIL_API int xmlrpc_response_fault_new_with_value (XmlrpcResponse **xrp,
			char *methodname,
			XmlrpcValue *xv);
CUTIL_API int xmlrpc_response_get_param_count (XmlrpcResponse *xr, int *paramcount);
CUTIL_API int xmlrpc_response_get_param (XmlrpcResponse *xr, int paramnumber,
			XmlrpcValue **xvp);
CUTIL_API int xmlrpc_response_get_methodname (XmlrpcResponse *xr, const char **methodnamep);
CUTIL_API int xmlrpc_response_serialize (XmlrpcResponse *xr, char **outputxmlp,
			int *outputxmllen);
CUTIL_API int xmlrpc_response_parse_xml (XmlrpcResponse **xrp, char *inputxml,
			int inputxmllen);
CUTIL_API int xmlrpc_response_free (XmlrpcResponse **xrp);


#endif
